#! /usr/bin/python
#-*- coding: utf-8 -*-
"""
Esegue un comando in un terminale e ne mostra l'output man mano che si
manifesta (purche' si utilizzi flush)

**EXAMPLES**

<code>

    eu-run-terminal  --command "ping 127.0.0.1"

</code>
"""
_required_ = ['eu-wcore']
import os
import sys
import subprocess
import termios
import argparse
import Tkinter


if os.name != 'nt':
    import fcntl


import imp
try:
    path = os.environ.get('EU_PATH','/usr/bin')
    CORE = imp.load_compiled('eu-wcore', os.path.join(path,'eu-wcore'))
except:
    print "No modules in EU_PATH %s" % path
    sys.exit(1)


NL = '\n'
SEP = '%s------%s' % (NL, NL)
FGCOLOR = '#17FA11'
TEMPFILE = "/tmp/termout.txt"


class RunCommand(CORE.BaseDialog):
    def on_init(self):
        if 'sudo' in self.args['command'].split():
            self.args['title'] = 'RUN IN TERMINAL AS SUPERUSER'
        else:
            self.args['title'] = 'RUN IN TERMINAL AS USER'
        if not self.args['legenda']:
            self.args['legenda'] = "CWD: %s" % self.args['cwd']
        self.args['font_size'] = 10
        self.p = None
        self.tkmess = Tkinter.StringVar()
        self.scroll = 0
        self.password = ''

    def on_body(self):
        f0 = Tkinter.Frame(self.bodyframe)
        f0.grid(row=0, column=0, padx=5, sticky='wn', pady=5)
        self.txt0 = CORE.TextScroll(f0)
        self.txt0.grid(row=0, column=0)
        self.txt0.configure(width=100, height=20, bg='black', fg=FGCOLOR,
                            font=self.widget_font_bold,
                            insertbackground='white', highlightthickness=1, insertwidth=5,
                            state='disabled')
        self.txt0.focus_set()
        self.txt0.tag_config('blue', foreground="blue")
        self.txt0.tag_config('yellow', foreground="yellow")
        self.txt0.tag_config('red', foreground="red")
        if self.args['command']:
            self.txt0.insert('end', self.args['command'])
        #----si preserva il comando originale dalle eventuali trasformazioni sudo
        self.args['orig_cmd'] = self.args['command']
        #----trasformazione sudo con richiesta password se necessaria
        self.do_prepare_sudocmd()

    def on_end(self):
        #----esecuzione
        self.lblgear.start()
        self.do_execute()

    def on_buttons(self):
        self.lblgear = CORE.LabelGear(parent=self.frame_bottom)
        self.addbutton(self.lblgear)
        lbl = Tkinter.Label(self.frame_bottom, textvariable=self.tkmess, width=20, bg='white', fg='red',
                            anchor='w', justify='left', font=self.widget_font_bold)
        self.addbutton(lbl)
        btn = CORE.ButtonChange(parent=self.frame_bottom, default=self.scroll, callback=self.do_scroll, text=['Scroll','NoScroll'])
        self.addbutton(btn)
        self.addbutton('Kill', self.do_terminate)
        self.addbutton('Save out', self.do_save)
        self.addbutton('Stdin', self.do_stdin)
        self.addbutton('Exit', self.root.destroy)

    def do_stdin(self):
        self.top = Tkinter.Toplevel(self.root)
        frame = Tkinter.Frame(self.top)
        frame.grid(row=0, column=0, padx=10, pady=10)
        lbl = Tkinter.Label(frame, text="Risposta:")
        lbl.grid(row=0, column=0)
        self.entry = Tkinter.Entry(frame, width=60, bg='white', relief='flat')
        self.entry.grid(row=0, column=1, padx=5)
        btn = Tkinter.Button(frame, text="Invia", command=self.do_invia)
        btn.grid(row=0, column=2)
        self.top.wm_attributes('-topmost', 1)

    def do_invia(self):
        buff = self.entry.get()
        if self.p:
            self.p.stdin.write(buff + '\n')
        self.top.destroy()

    def do_prepare_sudocmd(self):
        """Se viene rilevata la parola sudo si provvede a richiedere la password
        ed a creare lo script che consente di lanciare il comando
        """
        if 'sudo' in self.args['command'].split():
            if not self.password:
                out = CORE.RunDialog('EntryPassword', title="SUDO PASSWORD")
                if not out.strip():
                    return
                self.password = out.strip()
                self.args['command'] = self.args['command'].replace('sudo','')
                self.args['command'] = CORE.SudoPrepare(self.args['command'], self.password)

    def do_execute(self):
        """Esegue il comando
        """
        cmd = self.args['command'].strip()
        self.stop = False
        widget = self.txt0
        widget.configure(state='normal')
        if not self.args['hidecommand']:
            widget.insert('end', self.args['orig_cmd'], ('yellow',))
            widget.insert('end',NL)
        out = ''
        try:
            self.p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                                        stderr=subprocess.STDOUT,
                                        stdin=subprocess.PIPE,
                                        cwd=self.args['cwd'])
            self.idx = 0
            self.read_output(self.p, widget)
        except OSError as error:
            widget.insert('end', str(error))
        widget.see('end')
        widget.configure(state='disabled')

    def read_output(self, p, widget):
        """Legge l'output
        """
        if self.stop:
            widget.insert('end',SEP, ('red',))
            widget.see('end')
            return
        if not self.p.poll():
            self.p.stdout.flush()
            if os.name != 'nt':
                line = self.not_block_read(self.p)
            else:
                line = p.stdout.read(1024)
            widget.configure(state='normal')
            widget.insert('end', line.replace('\r\n','\n'))
            widget.update_idletasks()
            widget.see('end')
        if self.p.poll() != None:
            self.p.stdout.flush()
            try:
                if os.name != 'nt':
                    line = self.not_block_read(self.p)
                else:
                    line = p.stdout.read(1024)
                widget.configure(state='normal')
                widget.insert('end', line.replace('\r\n','\n'))
                widget.insert('end',SEP, ('red',))
                widget.see('end')
            except:
                widget.insert('end',SEP, ('red',))
                widget.see('end')
            self.tkmess.set("Executed")
            self.lblgear.stop()
            widget.see('end')
            return
        if self.scroll:
            widget.see('end')
        self.root.after(100, lambda x=p, y=widget: self.read_output(x,y))

    def not_block_read(self, p):
        if os.name != 'nt':
            fd = p.stdout.fileno()
            fl = fcntl.fcntl(fd, fcntl.F_GETFL)
            fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
        try:
            return p.stdout.read(1024)
        except IOError:
            return ""

    def do_terminate(self):
        self.stop = True
        if self.p:
            self.p.terminate()
            self.p = None

    def do_save(self):
        buff = self.txt0.get(1.0,'end')
        if isinstance(buff, unicode):
            buff = buff.encode('utf-8')
        open(self.args['outfile'], 'wb').write(buff)

    def do_scroll(self, value):
        self.scroll = value


def main():
    description = """eu-run-terminal"""
    epilog="Bye, bye."
    parser = argparse.ArgumentParser(prog=sys.argv[0], description=description, epilog=epilog, formatter_class=CORE.argparse_formatter())
    CORE.dialog_arguments(parser)
    parser.add_argument('--command', default="", help="Comando passato per l'esecuzione.")
    parser.add_argument('--cwd', default=os.getcwd(), help="Directory di esecuzone.")
    parser.add_argument('--hidecommand', action='store_true', help="Nasconde il comando e mostra solo l'output")
    parser.add_argument('--outfile', default=TEMPFILE, help="File dove salva l'output.")
    args = vars(parser.parse_args())
    if not args['command']:
        sys.exit("Missing command to execute")
    RunCommand(args)


if __name__ == '__main__':
    main()

